// Copyright (C) 1996 DIMACS Center, Rutgers, The State University of New Jersey
// Author(s): Jonathan Berry

// This software is copyrighted by the DIMACS Center at Rutgers, The State
// University of New Jersey.  IT IS PROVIDED AS IS, AND THE AUTHORS, DIMACS, AND
// RUTGERS, THE STATE UNIVERSITY OF NEW JERSEY  DISCLAIM
// ALL LIABILITY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
// DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE, ITS DOCUMENTATION, OR ANY
// DERIVATIVES THEREOF, EVEN IF THE AUTHORS HAVE BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.

// THE AUTHORS AND DISTRIBUTORS SPECIFICALLY DISCLAIM ANY WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT.  THIS SOFTWARE
// IS PROVIDED ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAVE
// NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
// MODIFICATIONS.

// The authors hereby grant permission to use, copy, modify, distribute,
// and license this software and its documentation for any purpose, provided
// that existing copyright notices are retained in all copies and that this
// notice is included verbatim in any distributions. No written agreement,
// license, or royalty fee is required for any of the authorized uses.
// Modifications to this software may be copyrighted by their authors
// and need not follow the licensing terms described here, provided that
// the new terms are clearly indicated on the first page of each file where
// they apply.

// Last File Update: 31-Jul-1996
// 

#ifndef UBinGraph_h
#define UBinGraph_h

#include <LINK/graph/BinGraph.h>
#include <LINK/graph/Vertex.h>

class Edge;

class MUBinGraph : public MBinGraph {
protected:

    Edge*		addEdge(Edge *e);

public:

    MUBinGraph() {}
    MUBinGraph(const Graph &G, Flag clone=CLONE, Flag reverse=0) 
				: MBinGraph(G, clone, reverse) {}

			// managing subgraphs
    Graph*              newGraph() { return (Graph*) new MUBinGraph; }
    Graph*              copyGraph(Flag clone = CLONE, Flag reverse = 0) 
			{ return (Graph*) new MUBinGraph(*this, clone,reverse);}

			// override for undirected binary graphs
    Edge*		addEdge(Sequence<Vertex*> vs, String n=0);
    Edge*		addEdge(Collection<Vertex*>& vs, String n=0);

    MSet<Edge*>		inIncidentEdges(Vertex* v)
				{ return v->incidentEdges(); }
    MSet<Edge*>		outIncidentEdges(Vertex* v)
				{ return v->incidentEdges(); }
    int			inDegree(Vertex* v) const
				{ return v->incidentEdges().size(); }
    int			outDegree(Vertex* v) const
				{ return v->incidentEdges().size(); }

    DataType		type() const	{ return M_UNDBINARYGRAPH; }
};

class UBinGraph : public MUBinGraph {
public:
    UBinGraph() {}
    UBinGraph(const Graph &G, Flag clone=CLONE, Flag reverse=0) 
				: MUBinGraph(G, clone, reverse) {}
    Graph*              newGraph() { return (Graph*) new UBinGraph; }
    Graph*              copyGraph(Flag clone = CLONE, Flag reverse = 0) 
			{ return (Graph*) new UBinGraph(*this, clone,reverse); }
    Edge*	addEdge(Collection<Vertex*>& vs, String n=0)
		   { return isEdge(vs) ? (Edge*)0 :MUBinGraph::addEdge(vs, n);}
    Edge*	addEdge(Sequence<Vertex*> vs, String n=0)
		   { return isEdge(vs) ? (Edge*)0 :MUBinGraph::addEdge(vs, n);}
    DataType	type() const	{ return UNDBINARYGRAPH; }
};


#endif
