// Copyright (C) 1996 DIMACS Center, Rutgers, The State University of New Jersey
// Author(s): Jonathan Berry

// This software is copyrighted by the DIMACS Center at Rutgers, The State
// University of New Jersey.  IT IS PROVIDED AS IS, AND THE AUTHORS, DIMACS, AND
// RUTGERS, THE STATE UNIVERSITY OF NEW JERSEY  DISCLAIM
// ALL LIABILITY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
// DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE, ITS DOCUMENTATION, OR ANY
// DERIVATIVES THEREOF, EVEN IF THE AUTHORS HAVE BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.

// THE AUTHORS AND DISTRIBUTORS SPECIFICALLY DISCLAIM ANY WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT.  THIS SOFTWARE
// IS PROVIDED ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAVE
// NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
// MODIFICATIONS.

// The authors hereby grant permission to use, copy, modify, distribute,
// and license this software and its documentation for any purpose, provided
// that existing copyright notices are retained in all copies and that this
// notice is included verbatim in any distributions. No written agreement,
// license, or royalty fee is required for any of the authorized uses.
// Modifications to this software may be copyrighted by their authors
// and need not follow the licensing terms described here, provided that
// the new terms are clearly indicated on the first page of each file where
// they apply.

// Last File Update: 31-Jul-1996
// 

#ifndef Vertex_h
#define Vertex_h

#include <LINK/graph/GraphObject.h>
#include <LINK/basic/Set.h>
#include <LINK/basic/general.h>

class Graph;
class Edge;
template <class Item> class MSet;


class Vertex : public GraphObject {
public:

    Vertex(Graph* owner);


    virtual ~Vertex();

			// manipulate incident edges to a vertex
    virtual void	addEdge(Edge* passed_edge);
    virtual void	removeEdge(Edge* passed_edge);

    //virtual Vertex*	copy();  Is ANYONE using this? -- mjd

    DataType		type() const		{ return VERTEX; }

    MSet<Edge*> 	incidentEdges() const	{ return _edges; }

    Set<Vertex*>	neighbors() 	const;
    Set<Vertex*>	inNeighbors() 	const;
    Set<Vertex*>	outNeighbors() 	const;

    String		name() const;	// override default

    Bool                operator<(const Vertex& obj)  const;
    Bool                operator<=(const Vertex& obj) const;
    Bool                operator>(const Vertex& obj)  const;
    Bool                operator>=(const Vertex& obj) const;
    Bool                operator==(const Vertex& obj) const;
    Bool                operator!=(const Vertex& obj) const;

    ostream&		display(ostream& stream) const;
    //friend ostream&	operator<<(ostream& stream, const Vertex& v);
    void		saveToFile(ofstream* fout, int indent);
    
private:
    friend class Graph;
    friend class MHyperGraph;
    friend class MBinGraph;
    Vertex(const Vertex &V, Flag clone=0);
    Set<Vertex*>	inNeighbors(Edge* passed_edge)  const;
    Set<Vertex*>	outNeighbors(Edge* passed_edge) const;

    MSet<Edge*>		_edges;
   
};


#endif
